package ssw.mj.test;

import ssw.mj.codegen.Code;

/**
 * Test cases for the examples from the lab sessions 8 and 9.
 */
public class UEBeispieleTest extends CompilerTestCase {
	/**
	 * Symbol table for all examples of this test class.
	 */
	private void expectSymTab() {
		expectSymTabUniverse();
		expectSymTab("Program A:");
		expectSymTab("  Constant: int max = 12");
		expectSymTab("  Global Variable 0: char c");
		expectSymTab("  Global Variable 1: int i");
		expectSymTab("  Type B: class(2)");
		expectSymTab("    Field 0: int x");
		expectSymTab("    Field 1: int y");
		expectSymTab("  Method: void main(0) at 0");
		expectSymTab("    Local Variable 0: int[] iarr");
		expectSymTab("    Local Variable 1: class(2) b");
		expectSymTab("    Local Variable 2: int n");
	}


	public void testBspEmpty() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);
		
		parseAndVerify();
	}

	public void testBsp01() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    n = 3;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.const_3);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);
		
		parseAndVerify();
	}
	
	public void testBsp02() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    i = 10;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.const_,    (int) 10);
		expectCode(EX >= 5, Code.putstatic, (short) 1);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);
		
		parseAndVerify();
	}
	
	public void testBsp03() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    n = 3 + i;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.const_3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.add);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);
		
		parseAndVerify();
	}
	
	public void testBsp04() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    n = 3 + i * max - n;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.const_3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.const_,    (int) 12);
		expectCode(EX >= 5, Code.mul);
		expectCode(EX >= 5, Code.add);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 5, Code.sub);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);
		
		parseAndVerify();
	}

	public void testBsp05() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    iarr = new int[10];" + LF +
		     "    iarr[5] = 10;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.const_,    (int) 10);
		expectCode(EX >= 5, Code.newarray,  (byte) 1);
		expectCode(EX >= 5, Code.store_0);
		expectCode(EX >= 5, Code.load_0);
		expectCode(EX >= 5, Code.const_5);
		expectCode(EX >= 5, Code.const_,    (int) 10);
		expectCode(EX >= 5, Code.astore);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);
		
		parseAndVerify();
	}
	
	public void testBsp06() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    iarr = new int[10];" + LF +
		     "    iarr[5] = 10;" + LF +
		     "    b = new B;" + LF +
		     "    b.y = iarr[5] * 3;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.const_,    (int) 10);
		expectCode(EX >= 5, Code.newarray,  (byte) 1);
		expectCode(EX >= 5, Code.store_0);
		expectCode(EX >= 5, Code.load_0);
		expectCode(EX >= 5, Code.const_5);
		expectCode(EX >= 5, Code.const_,    (int) 10);
		expectCode(EX >= 5, Code.astore);
		expectCode(EX >= 5, Code.new_,      (short) 8);
		expectCode(EX >= 5, Code.store_1);
		expectCode(EX >= 5, Code.load_1);
		expectCode(EX >= 5, Code.load_0);
		expectCode(EX >= 5, Code.const_5);
		expectCode(EX >= 5, Code.aload);
		expectCode(EX >= 5, Code.const_3);
		expectCode(EX >= 5, Code.mul);
		expectCode(EX >= 5, Code.putfield,  (short) 1);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}

	public void testBsp07() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    n--;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.inc,       (byte) 2, (byte) -1);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	
	
	public void testBsp08() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    i--;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.const_m1);
		expectCode(EX >= 5, Code.add);
		expectCode(EX >= 5, Code.putstatic, (short) 1);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	
	
	public void testBsp09() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    b = new B;" + LF +
		     "    b.y--;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.new_,      (short) 8);
		expectCode(EX >= 5, Code.store_1);
		expectCode(EX >= 5, Code.load_1);
		expectCode(EX >= 5, Code.dup);
		expectCode(EX >= 5, Code.getfield,  (short) 1);
		expectCode(EX >= 5, Code.const_m1);
		expectCode(EX >= 5, Code.add);
		expectCode(EX >= 5, Code.putfield,  (short) 1);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	

	public void testBsp10() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    iarr = new int[10];" + LF +
		     "    iarr[0]--;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.const_,    (int) 10);
		expectCode(EX >= 5, Code.newarray,  (byte) 1);
		expectCode(EX >= 5, Code.store_0);
		expectCode(EX >= 5, Code.load_0);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 5, Code.dup2);
		expectCode(EX >= 5, Code.aload);
		expectCode(EX >= 5, Code.const_m1);
		expectCode(EX >= 5, Code.add);
		expectCode(EX >= 5, Code.astore);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	

	public void testBsp11() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    if (i <= n) n = 0;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 6, Code.jgt,       (short) 5);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	

	public void testBsp12() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    if (i <= n && n < 0) n = 0;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 6, Code.jgt,       (short) 10);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 6, Code.jge,       (short) 5);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	

	public void testBsp13() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    if (i <= n || n < 0) n = 0;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 6, Code.jle,       (short) 8);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 6, Code.jge,       (short) 5);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	

	public void testBsp14() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    if (i <= n || n < 0 && i > 0) n = 0;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 6, Code.jle,       (short) 15);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 6, Code.jge,       (short) 12);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 6, Code.jle,       (short) 5);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	

	public void testBsp15() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    while (i <= n) n++;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 6, Code.jgt,       (short) 9);
		expectCode(EX >= 5, Code.inc,       (byte) 2, (byte) 1);
		expectCode(EX >= 6, Code.jmp,       (short) -10);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}	

	public void testBsp16() {
		init("program A" + LF +
		     "  final int max = 12;" + LF +
		     "  char c; int i;" + LF +
		     "  class B { int x, y; }" + LF +
		     "{" + LF +
		     "  void main ()" + LF +
		     "    int[] iarr; B b; int n;" + LF +
		     "  {" + LF +
		     "    if (i <= n) n = 0; else n = 1;" + LF +
		     "  }" + LF +
		     "}");

		expectSymTab();

		expectCode(EX >= 6, Code.enter,     (byte) 0, (byte) 3);
		expectCode(EX >= 5, Code.getstatic, (short) 1);
		expectCode(EX >= 5, Code.load_2);
		expectCode(EX >= 6, Code.jgt,       (short) 8);
		expectCode(EX >= 5, Code.const_0);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.jmp,       (short) 5);
		expectCode(EX >= 5, Code.const_1);
		expectCode(EX >= 5, Code.store_2);
		expectCode(EX >= 6, Code.exit);
		expectCode(EX >= 6, Code.return_);
		expectDataSize(2);
		expectMainPC(0);

		parseAndVerify();
	}
}
